;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :openair)

(progn
  (declaim (optimize (debug 3) (speed 0) (safety 1) (compilation-speed 0))))

(defmd html (family)
  id 
;;;  (jq-id? nil :cell nil
;;;    :documentation "Do we include this node's id in jq selectors?")
;;;  (jq-selector (c? (when (jq-id? self)
;;;                     (let ((s (jq-id self)))
;;;                       (bif (p (jq-path-through .parent))
;;;                         (conc$ p " > " s)
;;;                         s)))))
  (class nil :accessor cls)
  title
  style
  (send t)
  dirtyp
  (xhtml nil :unchanged-if 'string=))

(defmethod md-awaken ((self html))
  (bwhen (app (u^ web-app))
    (setf (gethash (id self) (nodes app)) self)
    ;;; hmmm.. got back "term" on request even tho openar.js says "id"....
    (setf (gethash (symbol-name (md-name self)) (nodes app)) self)))

(defmethod not-to-be :before ((self html))
  (bwhen (app (u^ web-app))
    (remhash (id self) (nodes app))
    (remhash (symbol-name (md-name self)) (nodes app))))

;;;(defobserver jq-selector ()
;;;  (when new-value
;;;    (trc "jqsel>" self new-value)))
;;;
;;;(defgeneric jq-path-through (self)
;;;  (:method (self) (declare (ignore self)) nil)
;;;  (:method ((self html))
;;;    (if (jq-id? self)
;;;        (jq-selector self)
;;;      (jq-path-through .parent))))

;;;(defun jq-id (self)
;;;  (let ((index (bwhen (id-parent (fm-ascendant-if self 'jq-id?))
;;;                 (let (my-pos (fm-traverse id-parent
;;;                   :global-search nil
;;;                   :opaque t
;;;                   :with-dependency t) when .parent
;;;                 (loop for k in (kids .parent)
;;;                       when (eq (md-name k) (md-name self))
;;;                       collect k into dups
;;;                       finally (when (cdr dups)
;;;                                 (return (position self dups)))))))
;;;    (if index
;;;        (format nil "#~(~a~)(~d)" (md-name self) index)
;;;      (format nil "#~(~a~)" (md-name self)))))

(defun js-escape (str)
  ;;  (print (list "escaping..." str))
  (with-output-to-string (s)
    (loop for char across str do
	 (case char
	   (#\' (princ "\\'" s))
	   (#\Newline nil)
	   (otherwise (princ char s))))))

(defmethod do-send (self new old)
  (bwhen (s (send self))
    (case s
      ((t) t)
      (:once (not old))
      (otherwise (funcall s self new old)))))

(defobserver xhtml ((self html))
  (bwhen (app (u^ web-app)) ;; not when playing around
    (when (and new-value (do-send self new-value old-value))
      ;(trc "xhtml do-send yes!" self)
      (let ((mdv (^value))
            (cd (gethash (^id) (client-side-data app))))
        (flet ((send-it ()
                 (trc "xhtml do-send and diff yes!" self)
                 (push (cons self
                         (format nil "$(\"#~a\").replaceWith(\"~a\")" (^id)
                           (js-escape new-value)))
                   (updates app))))
          (if (or mdv cd)
              (when (or (null mdv)(null cd)
                      (string/= cd mdv))
                (setf (gethash (^id) (client-side-data app)) mdv)
                (send-it))
            (send-it)))))))

;;; Grouping
(defhtml div () ())
(defhtml span () ())

;;; Headers
(defhtml h1 () ())
(defhtml h2 () ())
(defhtml h3 () ())

;;; Text

(defmodel text (html)
  ()
  (:default-initargs
      :xhtml (c? (with-output-to-string (s)
                   (with-html-output (s nil :prologue nil)
                     (:span :id (^id)
                       (str (^value))))))))

(defmacro mk-text (text)
  `(make-instance
    'text
    :fm-parent *parent*
    :id (gensym "S")
    :value ,text))

(defmodel page (html)
  ()
  (:default-initargs
      :xhtml (c? (trc "page xhtml rule begins!!!!" self .cause)
               (with-output-to-string (s)
                   (with-html-output (s nil :prologue t :indent t)
                     (:html
                      (:head
                       (:title (str (^title)))
                       (:script :type "text/javascript"
                         :src "/js/jquery.js")
                       (:script :text "text/javascript"
                         :src "/js/openair.js")
                       (:style :type "text/css"
                         :href (^style)))
                      (:body
                       (:div :id "banner"
                         (:h1 (str (^title))))
                       (:div :id "content"
                         (str (apply #'concatenate 'string
                                (loop for k in (^kids)
                                    collecting (without-c-dependency 
                                                   (xhtml k)))))))))))))
