;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :openair)

(progn
  (declaim (optimize (debug 3) (speed 0) (safety 1) (compilation-speed 0))))

;; Helpers

(defun defslot (name)
  `(,name :initform nil
	  :initarg ,(intern (string name) :keyword)))

(defun xmlrule (class html-attrs)
  `(c? (with-output-to-string (s)
         (with-html-output (s nil :prologue nil :indent t)
           (,(intern (symbol-name class) :keyword)
             ,@(loop for (cl-meth attr) in html-attrs
                   append `(,(intern (symbol-name attr) :keyword)
                             (,cl-meth self)))
             ,@(loop for (cl-meth attr) in '((id id) (cls class) (title title) (style style))
                   append `(,(intern (symbol-name attr) :keyword)
                             (,cl-meth self)))
             (str (apply #'concatenate 'string
                    (loop for k in (^kids)
                          collecting (without-c-dependency 
                                         (xhtml k))))))))))

(defun options-normalize (options)
  (loop for option in options
      for slot-name = (intern (string (if (listp option)
					  (car option)
					  option)))
      collecting (list slot-name (if (listp option)
				     (cadr option)
				     option))))

(defun define-model (class superclasses std-slots html-attrs defclass-options)
  `(defmodel ,class ,(or superclasses '(html))
     (,@(append std-slots
          (loop for (slot-name nil) in html-attrs
              collecting (defslot (intern (symbol-name slot-name))))))
     ,@(remove-if (lambda (k)
                    (find k '(:default-initargs :attrs)))
         defclass-options
         :key 'car)
     (:default-initargs
         ,@(cdr (find :default-initargs defclass-options :key 'car))
       :xhtml ,(xmlrule class html-attrs))))

(defun define-mk (class)
  `(defmacro ,(intern (concatenate
                          'string "MK-" (symbol-name class)))
     ((&rest inits) &body body)
     `(make-instance
          ',',class
        ,@inits
        :fm-parent *parent*
        :id (string (gensym (subseq (string ',',class) 0 1)))
        :kids (c? (the-kids
                   ,@(loop for kid in body
                         when (stringp kid)
                         collect `(mk-text ,kid)
                         else
                         collect kid))))))

(defmacro defhtml (class superclasses
		   (&rest std-slots)
		   &body defclass-options)
  (destructuring-bind (&optional html-class &rest html-attrs)
      (cdr (find :attrs defclass-options :key 'car))
    (declare (ignore html-class))

    (setf html-attrs (options-normalize html-attrs))

    `(progn
       ,(define-model class superclasses std-slots html-attrs defclass-options)
       ,(define-mk class)
       (find-class ',class))))

#| vestigial?


(defmacro mk-page ((&rest inits) &body body)
  `(make-instance 
    'page
    ,@inits
    :fm-parent *parent*
    :kids (c? (the-kids 
	       ,@body))))


(defmacro mk-div ((&rest inits) &body body)
  `(make-instance
    'div
    ,@inits
    :fm-parent *parent*
    :kids (c? (the-kids
	       ,@(loop for kid in body
		      when (stringp kid)
		       collect `(mk-text ,kid)
		      else
		       collect kid)))))


(defun dirty-rule (html-attrs)
  `(c? (let ((txt-kids
	      (without-c-dependency
		;;without dependency tracking because we don't want to
		;;let cells know we're reading all the kids, just the
		;;text ones
		(loop for kid in (^kids)
		     when (equal (class-of kid) 'text)
		       collect kid))))
	 ;;all we're doing here is establishing a dependancy between
	 ;;dirtyp and html attrs and text kids.  If this rule runs
	 ;;it should set dirtyp to t.
	 (map nil 'value txt-kids)
	 (list ,@(loop for (cl-meth nil) in html-attrs
		      collect `(,(intern (format nil "^~a" 
						 (symbol-name cl-meth))))))
	 t)))

(defun content-rule (html-attrs)
  `(c? (let ((content (remove nil
			      (list 
			       ,@(loop for (cl-meth nil) in html-attrs
				    collect `(,(intern (format nil "^~a"
							       (symbol-name cl-meth)))))
			       (loop for kid in (^kids)
				  when (eq (class-of kid) 'text)
				  append (value kid))))))
	 (print content)
	 content)))
		     
	       
	 
       
|#
