;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-

(defpackage #:openair-asd
  (:use :cl :asdf))

(in-package :openair-asd)

(progn
  (declaim (optimize (debug 3) (speed 0) (safety 1) (compilation-speed 0))))

;;; Openair requires a web-server to process requests and send
;;; responses.  I use hunchentoot but we'd like to make it also
;;; work with allegroserve.  You can choose which one you use
;;; by adding

(defun prompt-select (question options)
  (format *query-io* "~a ~%" question)
  (loop for opt in options
       for n upfrom 1 do
       (format *query-io*
	       "~a. ~a~%~^"
	       n opt))
  (force-output *query-io*)
  (nth (- (parse-integer (read-line *query-io*)) 1)
       options))

#+(and :use-hunch
       :use-aserve)
(case (prompt-select "Select Webserver" '(:hunchentoot :aserve))
  (:hunchentoot (setf *features* (remove :use-aserve *features*)))
  (:aserve (setf *features* (remove :use-hunch *features*))))

#-(or :use-hunch
      :use-aserve)
(case (prompt-select "Select Webserver" '(:hunchentoot :aserve))
  (:hunchentoot (pushnew :use-hunch *features*))
  (:aserve (pushnew :use-aserve *features*)))


#+:use-hunch
(defsystem :openair
  :name "openair"
  :version "0.0.1"
  :maintainer "Andy Chambers"
  :author "Andy Chambers"
  :licence "LLGPL"
  :description "A Common Lisp web framework."
  :depends-on (:hunchentoot :cl-who :cells :utils-kt)
  :components ((:file "package")
	       (:file "macros" :depends-on ("package"))
	       (:file "html" :depends-on ("macros"))
	       (:file "forms" :depends-on ("html"))
	       (:file "lists-tables" :depends-on ("html"))
	       (:file "request" :depends-on ("html"))
	       (:file "example" :depends-on ("request" "forms"))
  ))

#+:use-aserve
(defsystem :openair
  :name "openair"
  :version "0.0.1"
  :maintainer "Andy Chambers"
  :author "Andy Chambers"
  :licence "LLGPL"
  :description "A Common Lisp web framework."
  :depends-on (:aserve :cl-who :cells :utils-kt)
  :components ((:file "package")
	       (:file "macros" :depends-on ("package"))
	       (:file "html" :depends-on ("macros"))
	       (:file "forms" :depends-on ("html"))
	       (:file "lists-tables" :depends-on ("html"))
	       (:file "specials" :depends-on ("package"))
	       (:file "session" :depends-on ("specials"))
	       (:file "request" :depends-on ("html"))
	       (:file "example" :depends-on ("request" "forms"))
  ))
