;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :openair)

(defvar *request-type* :serverless)

(defun path (request)
  "returns the \"path\" part of the request (abstracts across different web-servers)"
  (case (type-of request)
    (cons (car request))
    (tbnl::request (script-name request))
    (t (error "not yet supported"))))

(defun get-param (self p)
  (bif (req (request (u^ web-app)))
     (case (type-of req)
       (cons (bogus-request-query-value p req))
       (tbnl::request (get-parameter p req))
       (t (error "not yet supported")))))

(defun bogus-request-query-value (p req)
  (bwhen (p (position p (cdr req) :test 'string=))
    (when (evenp p)
      (nth (1+ p) (cdr req)))))


(defmacro param? (name)
  `(or (get-param self ,name) .cache))

(defun call-param? (self p) ;; break out so do not have to recompile all while debugging and trying lots of things
  (bif (req (request (u^ web-app)))
    #+hunchentoot (get-parameter p req)
    #+aserve (bogus-request-query-value p req)))

(defun starts-with (str prefix)
  ;; (starts-with "ab" "a")
  ;; (starts-with "a" "a")
  ;; (starts-with "b" "a")
  (if (<= (length prefix) (length str))
      (equal (subseq str 0 (length prefix))
	     prefix)))

#+aserve 
(progn
  (defun script-name (request)
    #+soon (uri-path (request-uri request))
    (car request))
  (defun bogus-request-query-value (p req)
    (bwhen (p (position p (cdr req) :test 'string=))
      (when (evenp p)
        (nth (1+ p) (cdr req)))))
  (defun (setf content-type) (value)
    (declare (ignorable value))))

(defmodel web-app (family)
  ((prefix :initarg :prefix :accessor prefix :initform nil)
   (request :initarg :request :accessor request :initform nil)
   (message :initarg :message :accessor message :initform nil)
   (nodes :accessor nodes :initform (make-hash-table :test 'equal))
   (client-side-data :accessor client-side-data
     :initform (make-hash-table :test 'equal))
   (updates :initarg :updates 
     :cell nil
     :accessor updates 
     :initform nil)
   (resource :initarg :resource
     :owning t  ;; makes resource behave like a kid slot
     :accessor resource 
     :initform nil)
   (handler :initarg :handler :accessor handler :initform nil))
  (:default-initargs
      :prefix ""
    :request (c-in nil)
    :message (c-in nil)
    :handler (c? (bwhen (request (^request))
                   (trc "handler rule sees request!" cells::*data-pulse-id* request)
                   (describe request)
                   (let ((name (path request)))
                     (trc "web-app handler > name!" name (^prefix))
                     (cond
                      ((equal name (^prefix))
                       #'(lambda ()
                           (trc "web-app handler > init")
                           (setf (updates self) nil)
                           (setf (content-type) "text/html; charset=us-ascii")
                           (xhtml (car (^resource)))))
                      (t ;; I am getting "/get" as the script name (starts-with name (^prefix))
                       #'(lambda ()
                            (trc "web-app handler >  next")
                            (setf (content-type) "text/plain; charset=us-ascii")
                            (with-output-to-string (s)
                              (let ((updt (loop for (h . xh) in (updates self)
                                                do (if (mdead h)
                                                       (trc "req upd ignoring dead" h (type-of h))
                                                     (trc "raw update for node" (id h) (type-of h) h))
                                              unless (or (mdead h)
                                                       (loop for (h2 . xh) in (updates self)
                                                           when (typep h2 'web-app)
                                                           do (break "web-app in updates ~a" (cons h2 xh))
                                                           thereis (unless (eq h h2)
                                                                     (when (fm-ascendant-p h2 h)
                                                                       (trc "suppressing redundant" h :seeing-ascendant h2 (id h2))
                                                                       t))))
                                              collect xh
                                                and do (trc "making non-redundant update of" (id h)))))
                                (trc "updating client" updt)
                                (format s "~{~a~^, ~}" updt)
                                (setf (updates self) nil)
                                s))))))))))

(defobserver request ((self web-app) request)
  ;  #+chill 
  (when request
    (trc "req observer sees request!!!!" cells::*data-pulse-id* (get-parameters request))
    (loop for (name . value) in (get-parameters request)
        for node = (gethash name (^nodes))
        if node do
          (let ((cd (gethash name (^client-side-data))))
            (unless (and cd (string= cd value))
              (setf (gethash name (^client-side-data)) value)
              (with-cc :client-to-lisp
                (trc "node" node (id node) (md-name node) (type-of node) :gets-value value)
                (setf (value node) value))))
        else do
          (trc "whoa!!! got value for unknown node" name :value value)
          (trc "known nodes" (loop for hk being the hash-keys of (^nodes)
                                   collecting hk))
          (describe request))))

(defmacro mk-web-app ((&rest rest) &body body)
  `(make-instance
    'web-app
    ,@rest
    :fm-parent nil
    :resource (c? (the-kids ,@body))))

;; (start-server :port 8080)


				      
			  
